//
//  AppDelegate.swift
//  Reader
//
//  Created by Shadowfacts on 10/29/21.
//

import UIKit
import WebKit
import OSLog
import Combine

@main
class AppDelegate: UIResponder, UIApplicationDelegate {
    
    private var cancellables = Set<AnyCancellable>()

    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?) -> Bool {
        swizzleWKWebView()
        
        Preferences.shared.objectWillChange
            .debounce(for: .milliseconds(250), scheduler: RunLoop.main)
            .sink { _ in
                Preferences.save()
            }
            .store(in: &cancellables)
        
        return true
    }

    // MARK: UISceneSession Lifecycle

    func application(_ application: UIApplication, configurationForConnecting connectingSceneSession: UISceneSession, options: UIScene.ConnectionOptions) -> UISceneConfiguration {
        let name: String
        #if targetEnvironment(macCatalyst)
        if options.userActivities.first?.activityType == NSUserActivity.preferencesType {
            name = "prefs"
        } else {
            name = "main"
        }
        #else
        name = "main"
        #endif
        return UISceneConfiguration(name: name, sessionRole: connectingSceneSession.role)
    }

    func application(_ application: UIApplication, didDiscardSceneSessions sceneSessions: Set<UISceneSession>) {
        // Called when the user discards a scene session.
        // If any sessions were discarded while the application was not running, this will be called shortly after application:didFinishLaunchingWithOptions.
        // Use this method to release any resources that were specific to the discarded scenes, as they will not return.
    }
    
    override func buildMenu(with builder: UIMenuBuilder) {
        if builder.system == .main {
            builder.insertSibling(UIMenu(options: .displayInline, children: [
                UIKeyCommand(title: "Preferences…", action: #selector(showPreferences), input: ",", modifierFlags: .command)
            ]), afterMenu: .about)
            
            var children = [UIMenuElement]()
            let accounts: [UIMenuElement] = LocalData.accounts.map { account in
                var title = account.instanceURL.host!
                if let port = account.instanceURL.port, port != 80 && port != 443 {
                    title += ":\(port)"
                }
                
                let state: UIAction.State
                if let activeScene = UIApplication.shared.connectedScenes.first(where: { $0.activationState == .foregroundActive }),
                   let sceneDelegate = activeScene.delegate as? SceneDelegate,
                   sceneDelegate.fervorController?.account?.id == account.id {
                    state = .on
                } else {
                    state = .off
                }
                return UIAction(title: title, attributes: [], state: state) { _ in
                    let activity = NSUserActivity.activateAccount(account)
                    let options = UIScene.ActivationRequestOptions()
                    #if targetEnvironment(macCatalyst)
                    options.collectionJoinBehavior = .disallowed
                    #endif
                    UIApplication.shared.requestSceneSessionActivation(nil, userActivity: activity, options: options, errorHandler: nil)
                }
            }
            children.append(UIMenu(options: .displayInline, children: accounts))
            children.append(UIAction(title: "Add Account...", handler: { _ in
                let activity = NSUserActivity.addAccount()
                let options = UIScene.ActivationRequestOptions()
                #if targetEnvironment(macCatalyst)
                options.collectionJoinBehavior = .disallowed
                #endif
                UIApplication.shared.requestSceneSessionActivation(nil, userActivity: activity, options: options, errorHandler: nil)
            }))
            let account = UIMenu(title: "Account", image: nil, identifier: nil, options: [], children: children)
            builder.insertSibling(account, afterMenu: .file)
        }
    }
    
    private func swizzleWKWebView() {
        let selector = Selector(("_updateScrollViewBackground"))
        var originalIMP: IMP?
        let imp = imp_implementationWithBlock({ (self: WKWebView) in
            if let originalIMP = originalIMP {
                let original = unsafeBitCast(originalIMP, to: (@convention(c) (WKWebView, Selector) -> Void).self)
                original(self, selector)
            } else {
                os_log(.error, "Missing originalIMP for -[WKWebView _updateScrollViewBackground], did WebKit change?")
            }
            
            self.scrollView.indicatorStyle = .default
            
        } as (@convention(block) (WKWebView) -> Void))
        originalIMP = class_replaceMethod(WKWebView.self, selector, imp, "v@:")
    }
    
    @objc private func showPreferences() {
        UIApplication.shared.requestSceneSessionActivation(nil, userActivity: .preferences(), options: nil, errorHandler: nil)
    }

}

